// EMA_RSI.mq4

extern double lotSize = 0.1;
extern int startTradingHour = 7;
extern int endTradingHour = 17;
extern int maxTrades = 3;
extern bool onlyCloseInProfit = true;
extern double closeProfitPercentage = 25;
extern int magicNumber = 1;

extern int	MA_Fast_Period = 10;
extern int	MA_Mid_Period = 25;
extern int	MA_Slow_Period = 50;

extern int	RSI_Period = 18;
extern int	RSI_Buy_Restrict = 60;
extern int	RSI_Sell_Restrict = 70;

// added **
extern int	Points_Restrict = 60;	// Restricted zone (pips to High for Buy signal, to Low for Sell)
extern int	Points_Period = 15;		// Time Frame for High/Low detecting. In minutes
extern int	Points_Bars = 2;			// Number of bars for High/Low detecting
// **


datetime lastTrade;
datetime profitClose;
bool hadProfit = false;
int signal = 50;
bool dummyResult;

// * added
double gd_One_Pip_Rate;
string gs_Prefix = "NE_";
void init() {gd_One_Pip_Rate = MathPow(10, Digits);}
// **

int start() {

   if (hadProfit) {
      if (TimeDayOfYear(Time[0]) == TimeDayOfYear(profitClose))
         return (0);
      hadProfit = false;
   }
     
   // Check Order Exits
   int cnt = 0;
   while (cnt<OrdersTotal()) {
      if (!fastEMAClose(cnt)) cnt++;
   }
   
   // check Profit
   if (hasProfit()) {
      closeAllFastEMATrades("PROFIT");
      hadProfit = true;
      profitClose = Time[0];
      return (0);      
   }
   
   // check Trading hours
   if (Hour() < startTradingHour || Hour() > endTradingHour) {
      closeAllFastEMATrades("TIME");
      return (0);
   }

   // Check Order Entries
   fastEMATrade();   
   return(0);
}

bool hasProfit() {
   int cnt;
   int total=OrdersTotal();
   double totalProfit = 0;
   for(cnt=0;cnt<total;cnt++) {
      dummyResult = OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      totalProfit += OrderProfit();
   }
   if (((AccountBalance() * closeProfitPercentage) / 100) < totalProfit)
      return (true);
   return (false);
}

// Checks to see if we should trade
void fastEMATrade() {
	if (countOfFastEMATrades() < maxTrades && lastTrade != Time[0]) {
		double ema10 = iMA(Symbol(), 0, MA_Fast_Period, 0, MODE_EMA, PRICE_CLOSE, 1);
		double ema25 = iMA(Symbol(), 0, MA_Mid_Period, 0, MODE_EMA, PRICE_CLOSE, 1);
		double ema50 = iMA(Symbol(), 0, MA_Slow_Period, 0, MODE_EMA, PRICE_CLOSE, 1);
		double d_RSI = iRSI(Symbol(), 0, RSI_Period, PRICE_CLOSE, 1);
/* changed */
		if (ema10 < ema25 && ema25 < ema50 && d_RSI > RSI_Sell_Restrict) {
			if(Distance_OK(
				Points_Bars,
				Points_Period,
				Points_Restrict / gd_One_Pip_Rate,
				false
			)) {
				dummyResult = OrderSend(Symbol(), OP_SELL, lotSize, Bid,3,0,0,"EMA Sell", magicNumber);
				lastTrade = Time[0];
				return;
				   {
   Print("Order Send failed, error # ", GetLastError() );
   }
				
			}
		} else if (ema10 > ema25 && ema25 > ema50 && d_RSI < RSI_Buy_Restrict) {
			if(Distance_OK(
				Points_Bars,
				Points_Period,
				Points_Restrict / gd_One_Pip_Rate
			)) {
				dummyResult = OrderSend(Symbol(), OP_BUY, lotSize, Ask,3,0,0,"EMA Buy", magicNumber);
				lastTrade = Time[0];
				return;
			}
		}
// **
	}
}

bool fastEMAClose(int cnt) {
   double ema10 = iMA(Symbol(), 0, MA_Fast_Period, 0, MODE_EMA, PRICE_CLOSE, 1);
   double ema25 = iMA(Symbol(), 0, MA_Mid_Period, 0, MODE_EMA, PRICE_CLOSE, 1);
   double ema50 = iMA(Symbol(), 0, MA_Slow_Period, 0, MODE_EMA, PRICE_CLOSE, 1);
   dummyResult = OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
   if (OrderMagicNumber() == magicNumber) {
      if (ema10 < ema25 && ema25 < ema50 && OrderType() == OP_BUY) {
         if (!onlyCloseInProfit || OrderProfit() > 0) {
            Print("Closing BUY @ " + OrderProfit() + ":ask="+Ask+":bid="+Bid);
            return (OrderClose(OrderTicket(), OrderLots(), Bid, 3, Blue));
         }
      } else if (ema10 > ema25 && ema25 > ema50 && OrderType() == OP_SELL) {
         if (!onlyCloseInProfit || OrderProfit() > 0) {      
            Print("Closing SELL @ " + OrderProfit() + ":ask="+Ask+":bid="+Bid);
            return (OrderClose(OrderTicket(), OrderLots(), Ask, 3, Blue));
         }
      }
   }
   return (false);
}

void closeAllFastEMATrades(string reason) {
   int cnt;
   for (cnt=OrdersTotal(); cnt>=0; cnt--) {
      dummyResult = OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if (OrderMagicNumber() == magicNumber) {
         if (OrderType() == OP_BUY) {
            Print(reason + " Closing BUY @ " + OrderProfit() + ":ask="+Ask+":bid="+Bid);
            dummyResult = OrderClose(OrderTicket(), OrderLots(), Bid, 3, Blue);
         } else if (OrderType() == OP_SELL) {
            Print(reason + " Closing SELL @ " + OrderProfit() + ":ask="+Ask+":bid="+Bid);
            dummyResult = OrderClose(OrderTicket(), OrderLots(), Ask, 3, Blue);
         }
      }
   }
}

int countOfFastEMATrades() {
   int trades = 0;
   for (int cnt=0; cnt<OrdersTotal(); cnt++) {
      dummyResult = OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if (OrderMagicNumber() == magicNumber) {
         trades++;
      }
   }
   return (trades);
}


// added **
bool Distance_OK(int i_Bars, int i_Period, double d_Trigger_Level, bool b_High=true) {
	if(b_High) {
		if(
			iHigh(Symbol(), i_Period, iHighest(Symbol(), i_Period, MODE_HIGH, i_Bars, 0))
			- Bid > d_Trigger_Level
		) return (true);
	} else {
		if(
			Bid - iLow(Symbol(), i_Period, iLowest(Symbol(), i_Period, MODE_LOW, i_Bars, 0))
			> d_Trigger_Level
		) return (true);
	}
	
	return(false);
}
// **

void Show_Info(int& ia_Info[]) {
	int
		i_Value = 0,
		iX_Pos = 10,
		iY_Pos = 5
	;
	string s_String = "";
	color c_Color = OrangeRed;
	
	// Fan
	if(ia_Info[0] > 0) {
		s_String = "Fan: Buy";
		c_Color = LightGreen;
	} else if(ia_Info[0] == -1) {
		s_String = "Fan: Sell";
		c_Color = LightGreen;
	} else s_String = "Fan: spaghetti";
	Label_Handle(gs_Prefix + " fan", s_String, "Arial", 10, c_Color, 3, iX_Pos, iY_Pos);
	// Trades count
	iY_Pos += 25;
	s_String = "";
	c_Color = OrangeRed;
	if(ia_Info[4] == -1) {
		s_String = "Trades count: OK";
		c_Color = LightGreen;
	} else if(ia_Info[0] > -10)
		s_String = "Trades count: not met (" + ia_Info[4] + ")";
	Label_Handle(gs_Prefix + " trades", s_String, "Arial", 10, c_Color, 3, iX_Pos, iY_Pos);
	// Need order for this setup?
	iY_Pos += 25;
	s_String = "";
	c_Color = OrangeRed;
	if(ia_Info[1] == 0) {
		s_String = "Last cross: Need order";
		c_Color = LightGreen;
	} else if(ia_Info[0] > -10 && ia_Info[4] == -1)
		s_String = "Last cross: Processed";
	Label_Handle(gs_Prefix + " orders", s_String, "Arial", 10, c_Color, 3, iX_Pos, iY_Pos);
	// RSI
	iY_Pos += 25;
	s_String = "";
	c_Color = OrangeRed;
	if(ia_Info[2] == 0) {
		s_String = "RSI: OK";
		c_Color = LightGreen;
	} else if(ia_Info[0] > -10 && ia_Info[4] == -1 && ia_Info[1] > -10)
		s_String = "RSI: not met";
	Label_Handle(gs_Prefix + " rsi", s_String, "Arial", 10, c_Color, 3, iX_Pos, iY_Pos);
	// Distance
	iY_Pos += 25;
	s_String = "";
	c_Color = OrangeRed;
	if(ia_Info[3] == 0) {
		s_String = "Distance: OK";
		c_Color = LightGreen;
	} else if(ia_Info[0] > -10 && ia_Info[4] == -1 && ia_Info[1] > -10 && ia_Info[2] > -10)
		s_String = "Distance: not met";
	Label_Handle(gs_Prefix + " distance", s_String, "Arial", 10, c_Color, 3, iX_Pos, iY_Pos);
}


void Label_Handle(string sLabelName, string sLabelText = "", string sLabelFontName = "Arial", int iLabelFontSize = 10, color cLabelFontColor = Bisque, int iCorner = 0, int iXpos = 0, int iYpos = 0, int iAngle = 0, int iWindow = 0, bool bBackground=false) {
	
	if(ObjectFind(sLabelName) != -1) ObjectDelete(sLabelName);
	if(sLabelText != "") {
		ObjectCreate(sLabelName, OBJ_LABEL, iWindow, 0, 0);
		ObjectSet(sLabelName, OBJPROP_CORNER, iCorner);
		ObjectSet(sLabelName, OBJPROP_XDISTANCE, iXpos);
		ObjectSet(sLabelName, OBJPROP_YDISTANCE, iYpos);
		ObjectSet(sLabelName, OBJPROP_ANGLE, iAngle);
		ObjectSet(sLabelName, OBJPROP_BACK, bBackground);
		ObjectSetText(sLabelName, sLabelText, iLabelFontSize, sLabelFontName, cLabelFontColor);
	}
}


int RemoveObjects(string sName="", bool bExact=false) {
	int iObjectID = ObjectsTotal();
	while(iObjectID > 0) {
		iObjectID--;
		if(sName == "") ObjectDelete(ObjectName(iObjectID)); 
		else {
			if(bExact) { 
				if(ObjectName(iObjectID) == sName) ObjectDelete(ObjectName(iObjectID)); // kill'em
			}
			else {
				if(StringFind(ObjectName(iObjectID), sName) != 0) continue; 
				ObjectDelete(ObjectName(iObjectID)); // kill'em
			}
		}
	}
	
	return(0);
}